from netgen.csg import *
from ngsolve import *

geo = CSGeometry()
br1 = OrthoBrick (Pnt(0,0,0), Pnt(1,1,1)).bc("outerbnd")
br2 = OrthoBrick (Pnt(0.2,0.2,0.2), Pnt(0.8, 0.8, 0.8)).bc("innerbnd")
geo.Add (br2.mat("inner"))
geo.Add ( (br1-br2).mat("outer") )

mesh = Mesh(geo.GenerateMesh(maxh=0.3))

fesVec = VectorH1(mesh, order=2, dim = 3)
fesCurl = HCurl(mesh, order=2, dim = 3)
fesDiv = HDiv(mesh, order=2, dim = 3)

gfuVec = GridFunction(fesVec)
gfuCurl = GridFunction(fesCurl)
gfuDiv = GridFunction(fesDiv)

cf = CoefficientFunction((x,0,0))
gfuVec.Set(cf)
gfuCurl.Set(cf)
gfuDiv.Set(cf)

bfvcf = BoundaryFromVolumeCF(cf)
bfvVec = BoundaryFromVolumeCF(gfuVec)
bfvCurl = BoundaryFromVolumeCF(gfuCurl)
bfvDiv = BoundaryFromVolumeCF(gfuDiv)

normals = CoefficientFunction ([None, specialcf.normal(3)] )
bfv_normals = BoundaryFromVolumeCF(func_domain2)

print("CF     Integral = ", Integrate(bfvcf*bfv_normals, mesh, definedon=mesh.Boundaries("innerbnd")))
print("H-Vec  Integral = ", Integrate(bfvVec*bfv_normals, mesh, definedon=mesh.Boundaries("innerbnd")))
print("H-Curl Integral = ", Integrate(bfvCurl*bfv_normals, mesh, definedon=mesh.Boundaries("innerbnd")))
print("H-Div  Integral = ", Integrate(bfvDiv*bfv_normals, mesh, definedon=mesh.Boundaries("innerbnd")))

